@@ -44,7 +44,8 @@
 
 /* Supported set of tuning options, within this implementation */
 #define SUPPORTED_TUNE_OPTIONS                          \
-  (GST_VAAPI_ENCODER_TUNE_MASK (NONE))
+  (GST_VAAPI_ENCODER_TUNE_MASK (NONE) |                 \
+   GST_VAAPI_ENCODER_TUNE_MASK (LOW_POWER))
 
 /* Supported set of VA packed headers, within this implementation */
 #define SUPPORTED_PACKED_HEADERS                \
@@ -92,6 +93,7 @@ struct _GstVaapiEncoderH265
   GstVaapiProfile profile;
   GstVaapiTierH265 tier;
   GstVaapiLevelH265 level;
+  GstVaapiEntrypoint entrypoint;
   guint8 profile_idc;
   guint8 max_profile_idc;
   guint8 hw_max_profile_idc;
@@ -918,7 +920,7 @@ static gboolean
 ensure_hw_profile (GstVaapiEncoderH265 * encoder)
 {
   GstVaapiDisplay *const display = GST_VAAPI_ENCODER_DISPLAY (encoder);
-  GstVaapiEntrypoint entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
+  GstVaapiEntrypoint entrypoint = encoder->entrypoint;
   GstVaapiProfile profile, profiles[4];
   guint i, num_profiles = 0;
 
@@ -996,38 +998,46 @@ ensure_profile (GstVaapiEncoderH265 * encoder)
   return TRUE;
 }
 
-/* Derives the minimum tier from the active coding tools */
+/* Derives the level and tier from the currently set limits */
 static gboolean
-ensure_tier (GstVaapiEncoderH265 * encoder)
+ensure_tier_level (GstVaapiEncoderH265 * encoder)
 {
-
-  encoder->tier = GST_VAAPI_TIER_H265_MAIN;
-  /* FIXME: Derive proper tier based on upstream caps or limits, coding
-   * tools etc */
-
-  return TRUE;
-}
-
-/* Derives the level from the currently set limits */
-static gboolean
-ensure_level (GstVaapiEncoderH265 * encoder)
-{
-  const GstVaapiH265LevelLimits *limits_table;
+  guint bitrate = GST_VAAPI_ENCODER_CAST (encoder)->bitrate;
   guint i, num_limits, PicSizeInSamplesY;
+  guint LumaSr;
+  const GstVaapiH265LevelLimits *limits_table;
 
   PicSizeInSamplesY = encoder->luma_width * encoder->luma_height;
+  LumaSr =
+      gst_util_uint64_scale (PicSizeInSamplesY,
+      GST_VAAPI_ENCODER_FPS_N (encoder), GST_VAAPI_ENCODER_FPS_D (encoder));
 
   limits_table = gst_vaapi_utils_h265_get_level_limits_table (&num_limits);
   for (i = 0; i < num_limits; i++) {
     const GstVaapiH265LevelLimits *const limits = &limits_table[i];
-    if (PicSizeInSamplesY <= limits->MaxLumaPs)
+    /* Choose level by luma picture size and luma sample rate */
+    if (PicSizeInSamplesY <= limits->MaxLumaPs && LumaSr <= limits->MaxLumaSr)
       break;
-    /* FIXME: Add more constraint checking:tier (extracted from caps),
-     * cpb size, bitrate, num_tile_columns and num_tile_rows */
   }
+
   if (i == num_limits)
     goto error_unsupported_level;
 
+  if (bitrate <= limits_table[i].MaxBRTierMain) {
+    encoder->tier = GST_VAAPI_TIER_H265_MAIN;
+  } else {
+    encoder->tier = GST_VAAPI_TIER_H265_HIGH;
+    if (bitrate > limits_table[i].MaxBRTierHigh) {
+      GST_INFO ("The bitrate of the stream is %d kbps, larger than"
+          " %s profile %s level %s tier's max bit rate %d kbps",
+          bitrate,
+          gst_vaapi_utils_h265_get_profile_string (encoder->profile),
+          gst_vaapi_utils_h265_get_level_string (limits_table[i].level),
+          gst_vaapi_utils_h265_get_tier_string (GST_VAAPI_TIER_H265_HIGH),
+          limits_table[i].MaxBRTierHigh);
+    }
+  }
+
   encoder->level = limits_table[i].level;
   encoder->level_idc = limits_table[i].level_idc;
   return TRUE;
@@ -1040,6 +1050,7 @@ error_unsupported_level:
   }
 }
 
+/* Derives the minimum tier from the active coding tools */
 /* Enable "high-compression" tuning options */
 static gboolean
 ensure_tuning_high_compression (GstVaapiEncoderH265 * encoder)
@@ -1070,6 +1081,10 @@ ensure_tuning (GstVaapiEncoderH265 * encoder)
     case GST_VAAPI_ENCODER_TUNE_HIGH_COMPRESSION:
       success = ensure_tuning_high_compression (encoder);
       break;
+    case GST_VAAPI_ENCODER_TUNE_LOW_POWER:
+      encoder->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP;
+      success = TRUE;
+      break;
     default:
       success = TRUE;
       break;
@@ -1525,9 +1540,12 @@ fill_sequence (GstVaapiEncoderH265 * encoder, GstVaapiEncSequence * sequence)
   seq_param->seq_fields.bits.sps_temporal_mvp_enabled_flag =
       encoder->sps_temporal_mvp_enabled_flag = TRUE;
 
-  /* Based on 32x32 CTU */
+  /* Based on 32x32 CTU (64x64 when using lowpower mode for hardware limitation) */
   seq_param->log2_min_luma_coding_block_size_minus3 = 0;
-  seq_param->log2_diff_max_min_luma_coding_block_size = 2;
+  if (GST_VAAPI_ENCODER_TUNE (encoder) == GST_VAAPI_ENCODER_TUNE_LOW_POWER)
+    seq_param->log2_diff_max_min_luma_coding_block_size = 3;
+  else
+    seq_param->log2_diff_max_min_luma_coding_block_size = 2;
   seq_param->log2_min_transform_block_size_minus2 = 0;
   seq_param->log2_diff_max_min_transform_block_size = 3;
   seq_param->max_transform_hierarchy_depth_inter = 3;
@@ -1622,9 +1640,10 @@ fill_picture (GstVaapiEncoderH265 * encoder, GstVaapiEncPicture * picture,
   pic_param->pic_fields.bits.sign_data_hiding_enabled_flag = FALSE;
   pic_param->pic_fields.bits.transform_skip_enabled_flag = TRUE;
   /* it seems driver requires enablement of cu_qp_delta_enabled_flag
-   * to modifiy QP values in CBR mode encoding */
-  pic_param->pic_fields.bits.cu_qp_delta_enabled_flag =
-      GST_VAAPI_ENCODER_RATE_CONTROL (encoder) != GST_VAAPI_RATECONTROL_CQP;
+   * to modifiy QP values in CBR mode or low power encoding */
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) != GST_VAAPI_RATECONTROL_CQP
+      || GST_VAAPI_ENCODER_TUNE (encoder) == GST_VAAPI_ENCODER_TUNE_LOW_POWER)
+    pic_param->pic_fields.bits.cu_qp_delta_enabled_flag = 1;
 
   pic_param->pic_fields.bits.pps_loop_filter_across_slices_enabled_flag = TRUE;
 
@@ -1848,6 +1867,10 @@ ensure_control_rate_params (GstVaapiEncoderH265 * encoder)
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).window_size = encoder->cpb_length;
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).initial_qp = encoder->init_qp;
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).min_qp = encoder->min_qp;
+  /* @FIXME: should not set this value, should be ignored if set to zero *
+   * https://github.com/intel/media-driver/issues/587 */
+  if (encoder->min_qp > 0)
+    GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).max_qp = 51;
 
 #if VA_CHECK_VERSION(1,0,0)
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).rc_flags.bits.mb_rate_control =
@@ -2010,14 +2033,10 @@ ensure_profile_tier_level (GstVaapiEncoderH265 * encoder)
   if (encoder->profile_idc > encoder->hw_max_profile_idc)
     return GST_VAAPI_ENCODER_STATUS_ERROR_UNSUPPORTED_PROFILE;
 
-  /* ensure tier */
-  if (!ensure_tier (encoder))
-    return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
-
   /* Ensure bitrate if not set already and derive the right level to use */
   ensure_bitrate (encoder);
 
-  if (!ensure_level (encoder))
+  if (!ensure_tier_level (encoder))
     return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
 
   if (encoder->profile != profile || encoder->level != level
@@ -2048,11 +2067,10 @@ reset_properties (GstVaapiEncoderH265 * encoder)
 
   ctu_size = encoder->ctu_width * encoder->ctu_height;
   g_assert (gst_vaapi_encoder_ensure_num_slices (base_encoder, encoder->profile,
-          GST_VAAPI_ENTRYPOINT_SLICE_ENCODE, (ctu_size + 1) / 2,
-          &encoder->num_slices));
+          encoder->entrypoint, (ctu_size + 1) / 2, &encoder->num_slices));
 
   gst_vaapi_encoder_ensure_max_num_ref_frames (base_encoder, encoder->profile,
-      GST_VAAPI_ENTRYPOINT_SLICE_ENCODE);
+      encoder->entrypoint);
 
   if (base_encoder->max_num_ref_frames_1 < 1 && encoder->num_bframes > 0) {
     GST_WARNING ("Disabling b-frame since the driver doesn't support it");
@@ -2505,6 +2523,8 @@ set_context_info (GstVaapiEncoder * base_encoder)
   base_encoder->codedbuf_size += GST_ROUND_UP_16 (vip->width) *
       GST_ROUND_UP_16 (vip->height) * 3 / 2;
 
+  base_encoder->context_info.entrypoint = encoder->entrypoint;
+
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
 }
 
@@ -2523,10 +2543,14 @@ gst_vaapi_encoder_h265_reconfigure (GstVaapiEncoder * base_encoder)
         GST_VAAPI_ENCODER_HEIGHT (encoder));
     encoder->luma_width = GST_ROUND_UP_16 (luma_width);
     encoder->luma_height = GST_ROUND_UP_16 (luma_height);
-    encoder->ctu_width = (encoder->luma_width + 31) / 32;
-    encoder->ctu_height = (encoder->luma_height + 31) / 32;
+    if (GST_VAAPI_ENCODER_TUNE (encoder) == GST_VAAPI_ENCODER_TUNE_LOW_POWER) {
+      encoder->ctu_width = (encoder->luma_width + 63) / 64;
+      encoder->ctu_height = (encoder->luma_height + 63) / 64;
+    } else {
+      encoder->ctu_width = (encoder->luma_width + 31) / 32;
+      encoder->ctu_height = (encoder->luma_height + 31) / 32;
+    }
     encoder->config_changed = TRUE;
-
     /* Frame Cropping */
     if ((GST_VAAPI_ENCODER_WIDTH (encoder) & 15) ||
         (GST_VAAPI_ENCODER_HEIGHT (encoder) & 15)) {
@@ -2560,6 +2584,9 @@ gst_vaapi_encoder_h265_init (GstVaapiEncoder * base_encoder)
   GstVaapiH265ReorderPool *reorder_pool;
   GstVaapiH265RefPool *ref_pool;
 
+  /* Default encoding entrypoint */
+  encoder->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
+
   encoder->conformance_window_flag = 0;
   encoder->num_slices = 1;
 
@@ -2737,7 +2764,7 @@ gst_vaapi_encoder_h265_get_default_properties (void)
   GST_VAAPI_ENCODER_PROPERTIES_APPEND (props,
       GST_VAAPI_ENCODER_H265_PROP_INIT_QP,
       g_param_spec_uint ("init-qp",
-          "Initial QP", "Initial quantizer value", 1, 51, 26,
+          "Initial QP", "Initial quantizer value", 0, 51, 26,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
@@ -2748,7 +2775,7 @@ gst_vaapi_encoder_h265_get_default_properties (void)
   GST_VAAPI_ENCODER_PROPERTIES_APPEND (props,
       GST_VAAPI_ENCODER_H265_PROP_MIN_QP,
       g_param_spec_uint ("min-qp",
-          "Minimum QP", "Minimum quantizer value", 1, 51, 1,
+          "Minimum QP", "Minimum quantizer value", 0, 51, 1,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
