@@ -28,7 +28,6 @@
  */
 
 #include "sysdeps.h"
-#include <string.h>
 #include "gstvaapicompat.h"
 #include "gstvaapiutils.h"
 #include "gstvaapiimage.h"
@@ -92,14 +91,17 @@ vaapi_image_is_linear (const VAImage * va_image)
     case VA_FOURCC ('R', 'G', 'B', 'X'):
     case VA_FOURCC ('X', 'B', 'G', 'R'):
     case VA_FOURCC ('B', 'G', 'R', 'X'):
+    case VA_FOURCC ('Y', '2', '1', '0'):
+    case VA_FOURCC ('Y', '4', '1', '0'):
       data_size = 4 * width * height;
       break;
     case VA_FOURCC ('P', '0', '1', '0'):
       data_size = 2 * (width * height + 2 * width2 * height2);
       break;
     default:
-      g_error ("FIXME: incomplete formats %" GST_FOURCC_FORMAT,
+      GST_ERROR ("FIXME: incomplete formats %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (va_image->format.fourcc));
+      data_size = G_MAXUINT;
       break;
   }
   return va_image->data_size == data_size;
@@ -122,7 +124,7 @@ gst_vaapi_image_destroy (GstVaapiImage * image)
     status = vaDestroyImage (GST_VAAPI_DISPLAY_VADISPLAY (display), image_id);
     GST_VAAPI_DISPLAY_UNLOCK (display);
     if (!vaapi_check_status (status, "vaDestroyImage()"))
-      g_warning ("failed to destroy image %" GST_VAAPI_ID_FORMAT,
+      GST_WARNING ("failed to destroy image %" GST_VAAPI_ID_FORMAT,
           GST_VAAPI_ID_ARGS (image_id));
     GST_VAAPI_OBJECT_ID (image) = VA_INVALID_ID;
   }
