@@ -214,38 +214,30 @@ string_of_VAProfile (VAProfile profile)
       MAP (MPEG4Simple);
       MAP (MPEG4AdvancedSimple);
       MAP (MPEG4Main);
-#if VA_CHECK_VERSION(0,32,0)
       MAP (JPEGBaseline);
       MAP (H263Baseline);
       MAP (H264ConstrainedBaseline);
-#endif
 #if !VA_CHECK_VERSION(1,0,0)
       MAP (H264Baseline);
 #endif
       MAP (H264Main);
       MAP (H264High);
-#if VA_CHECK_VERSION(0,35,2)
       MAP (H264MultiviewHigh);
       MAP (H264StereoHigh);
+#if VA_CHECK_VERSION(1,2,0)
+      MAP (HEVCMain422_10);
+      MAP (HEVCMain444);
 #endif
-#if VA_CHECK_VERSION(0,37,1)
       MAP (HEVCMain);
       MAP (HEVCMain10);
-#endif
       MAP (VC1Simple);
       MAP (VC1Main);
       MAP (VC1Advanced);
-#if VA_CHECK_VERSION(0,35,0)
       MAP (VP8Version0_3);
-#endif
-#if VA_CHECK_VERSION(0,37,0)
       MAP (VP9Profile0);
-#endif
-#if VA_CHECK_VERSION(0,39,0)
       MAP (VP9Profile1);
       MAP (VP9Profile2);
       MAP (VP9Profile3);
-#endif
 #undef MAP
     default:
       break;
@@ -284,17 +276,8 @@ string_of_VADisplayAttributeType (VADisplayAttribType attribute_type)
       MAP (Hue);
       MAP (Saturation);
       MAP (BackgroundColor);
-#if !VA_CHECK_VERSION(0,34,0)
-      MAP (DirectSurface);
-#endif
       MAP (Rotation);
       MAP (OutofLoopDeblock);
-#if VA_CHECK_VERSION(0,31,1) && !VA_CHECK_VERSION(0,34,0)
-      MAP (BLEBlackMode);
-      MAP (BLEWhiteMode);
-      MAP (BlueStretch);
-      MAP (SkinColorCorrection);
-#endif
       MAP (CSCMatrix);
       MAP (BlendColor);
       MAP (OverlayAutoPaintColorKey);
@@ -319,14 +302,18 @@ string_of_va_chroma_format (guint chroma_format)
       MAP (YUV420);
       MAP (YUV422);
       MAP (YUV444);
-#if VA_CHECK_VERSION(0,34,0)
       MAP (YUV400);
       MAP (RGB16);
       MAP (RGB32);
       MAP (RGBP);
-#endif
-#if VA_CHECK_VERSION(0,38,1)
       MAP (YUV420_10BPP);
+#if VA_CHECK_VERSION(1,2,0)
+      MAP (YUV422_10);
+      MAP (YUV444_10);
+      MAP (YUV420_12);
+      MAP (YUV422_12);
+      MAP (YUV444_12);
+      MAP (RGB32_10);
 #endif
 #undef MAP
     default:
@@ -385,7 +372,6 @@ to_GstVaapiChromaType (guint va_rt_format)
     case VA_RT_FORMAT_YUV444:
       chroma_type = GST_VAAPI_CHROMA_TYPE_YUV444;
       break;
-#if VA_CHECK_VERSION(0,34,0)
     case VA_RT_FORMAT_YUV411:
       chroma_type = GST_VAAPI_CHROMA_TYPE_YUV411;
       break;
@@ -398,11 +384,31 @@ to_GstVaapiChromaType (guint va_rt_format)
     case VA_RT_FORMAT_RGB16:
       chroma_type = GST_VAAPI_CHROMA_TYPE_RGB16;
       break;
-#endif
-#if VA_CHECK_VERSION(0,38,1)
+    case VA_RT_FORMAT_RGBP:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_RGBP;
+      break;
     case VA_RT_FORMAT_YUV420_10BPP:
       chroma_type = GST_VAAPI_CHROMA_TYPE_YUV420_10BPP;
       break;
+#if VA_CHECK_VERSION(1,2,0)
+    case VA_RT_FORMAT_YUV422_10:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_YUV422_10BPP;
+      break;
+    case VA_RT_FORMAT_YUV444_10:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_YUV444_10BPP;
+      break;
+    case VA_RT_FORMAT_YUV420_12:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_YUV420_12BPP;
+      break;
+    case VA_RT_FORMAT_YUV422_12:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_YUV422_12BPP;
+      break;
+    case VA_RT_FORMAT_YUV444_12:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_YUV444_12BPP;
+      break;
+    case VA_RT_FORMAT_RGB32_10:
+      chroma_type = GST_VAAPI_CHROMA_TYPE_RGB32_10BPP;
+      break;
 #endif
     default:
       chroma_type = 0;
@@ -433,7 +439,6 @@ from_GstVaapiChromaType (guint chroma_type)
     case GST_VAAPI_CHROMA_TYPE_YUV444:
       format = VA_RT_FORMAT_YUV444;
       break;
-#if VA_CHECK_VERSION(0,34,0)
     case GST_VAAPI_CHROMA_TYPE_YUV411:
       format = VA_RT_FORMAT_YUV411;
       break;
@@ -446,11 +451,31 @@ from_GstVaapiChromaType (guint chroma_type)
     case GST_VAAPI_CHROMA_TYPE_RGB16:
       format = VA_RT_FORMAT_RGB16;
       break;
-#endif
-#if VA_CHECK_VERSION(0,38,1)
+    case GST_VAAPI_CHROMA_TYPE_RGBP:
+      format = VA_RT_FORMAT_RGBP;
+      break;
     case GST_VAAPI_CHROMA_TYPE_YUV420_10BPP:
       format = VA_RT_FORMAT_YUV420_10BPP;
       break;
+#if VA_CHECK_VERSION(1,2,0)
+    case GST_VAAPI_CHROMA_TYPE_YUV422_10BPP:
+      format = VA_RT_FORMAT_YUV422_10;
+      break;
+    case GST_VAAPI_CHROMA_TYPE_YUV444_10BPP:
+      format = VA_RT_FORMAT_YUV444_10;
+      break;
+    case GST_VAAPI_CHROMA_TYPE_YUV420_12BPP:
+      format = VA_RT_FORMAT_YUV420_12;
+      break;
+    case GST_VAAPI_CHROMA_TYPE_YUV422_12BPP:
+      format = VA_RT_FORMAT_YUV422_12;
+      break;
+    case GST_VAAPI_CHROMA_TYPE_YUV444_12BPP:
+      format = VA_RT_FORMAT_YUV444_12;
+      break;
+    case GST_VAAPI_CHROMA_TYPE_RGB32_10BPP:
+      format = VA_RT_FORMAT_RGB32_10;
+      break;
 #endif
     default:
       format = 0;
@@ -625,10 +650,8 @@ to_GstVaapiSurfaceStatus (guint va_flags)
   }
 
   /* Check for encoder status */
-#if VA_CHECK_VERSION(0,30,0)
   if (va_flags & VASurfaceSkipped)
     flags |= GST_VAAPI_SURFACE_STATUS_SKIPPED;
-#endif
   return flags;
 }
 
@@ -733,7 +756,6 @@ from_GstVaapiDeinterlaceMethod (guint value)
   switch (value) {
     case GST_VAAPI_DEINTERLACE_METHOD_NONE:
       return 0;
-#if USE_VA_VPP
     case GST_VAAPI_DEINTERLACE_METHOD_BOB:
       return VAProcDeinterlacingBob;
     case GST_VAAPI_DEINTERLACE_METHOD_WEAVE:
@@ -742,7 +764,6 @@ from_GstVaapiDeinterlaceMethod (guint value)
       return VAProcDeinterlacingMotionAdaptive;
     case GST_VAAPI_DEINTERLACE_METHOD_MOTION_COMPENSATED:
       return VAProcDeinterlacingMotionCompensated;
-#endif
   }
   GST_ERROR ("unsupported GstVaapiDeinterlaceMethod value %d", value);
   return 0;
@@ -754,7 +775,6 @@ from_GstVaapiDeinterlaceFlags (guint flags)
 {
   guint va_flags = 0;
 
-#if USE_VA_VPP
   if (!(flags & GST_VAAPI_DEINTERLACE_FLAG_TFF))
     va_flags |= VA_DEINTERLACING_BOTTOM_FIELD_FIRST;
 
@@ -763,7 +783,6 @@ from_GstVaapiDeinterlaceFlags (guint flags)
 
   if (!(flags & GST_VAAPI_DEINTERLACE_FLAG_TOPFIELD))
     va_flags |= VA_DEINTERLACING_BOTTOM_FIELD;
-#endif
   return va_flags;
 }
 
